import numpy as np
import pandas as pd
import pytest
from abagen import datasets

KEYS = [
    'microarray', 'annotation', 'pacall', 'probes', 'ontology'
]


def test_fetch_datasets(testdir):
    # check downloading for a subset of donors
    files = datasets.fetch_microarray(data_dir=str(testdir),
                                      donors=['12876'])
    assert isinstance(files, dict)
    for k in KEYS:
        assert len(files.get(k)) == 1

    # check downloading incorrect donor
    with pytest.raises(ValueError):
        datasets.fetch_microarray(donors=['notadonor'])

    files = datasets.fetch_microarray(data_dir=str(testdir),
                                      donors=None)


def test_fetch_alleninf_coords():
    coords = datasets._fetch_alleninf_coords()
    assert isinstance(coords, pd.DataFrame)
    assert coords.index.name == 'well_id'
    assert np.all(coords.columns == ['mni_x', 'mni_y', 'mni_z'])
    assert coords.shape == (3702, 3)


def test_fetch_mri():
    with pytest.raises(NotImplementedError):
        datasets.fetch_mri()



import os.path as op
import numpy as np
import pandas as pd
import pytest
from abagen import io


IO_FUNC = [
    dict(
        key='microarray',
        has_parq=True,
        columns=None,
        func=io.read_microarray
    ),
    dict(
        key='pacall',
        has_parq=True,
        columns=None,
        func=io.read_pacall
    ),
    dict(
        key='annotation',
        has_parq=False,
        columns=[
            'structure_id', 'slab_num', 'well_id', 'slab_type',
            'structure_acronym', 'structure_name', 'polygon_id', 'mri_voxel_x',
            'mri_voxel_y', 'mri_voxel_z', 'mni_x', 'mni_y', 'mni_z'
        ],
        func=io.read_annotation
    ),
    dict(
        key='ontology',
        has_parq=False,
        columns=[
            'id', 'acronym', 'name', 'parent_structure_id', 'hemisphere',
            'graph_order', 'structure_id_path', 'color_hex_triplet'
        ],
        func=io.read_ontology
    ),
    dict(
        key='probes',
        has_parq=False,
        columns=[
            'probe_name', 'gene_id', 'gene_symbol', 'gene_name', 'entrez_id',
            'chromosome'
        ],
        func=io.read_probes
    )
]


def test_readfiles(testfiles):
    for config in IO_FUNC:
        for fn in testfiles.get(config['key']):
            assert op.exists(fn)
            if config['has_parq'] and io.use_parq:
                assert op.exists(fn.rpartition('.csv')[0] + '.parq')

            # check loading from filepath
            data = config['func'](fn, parquet=True)
            assert isinstance(data, pd.DataFrame)

            # check loading from dataframe
            data = config['func'](data)
            assert isinstance(data, pd.DataFrame)

            if config['columns'] is not None:
                assert np.all(config['columns'] == data.columns)

        with pytest.raises(TypeError):
            config['func']([1, 2, 3])



# -*- coding: utf-8 -*-
"""
Functions for downloading Allen Brain Atlas human microarray dataset

Modeled after ``nilearn.datasets``. Currently just downloads into current
working directory, but will likely be modified to download into a more
"standard" directory.
"""

from io import StringIO
import os
from pkg_resources import resource_filename
from nibabel.volumeutils import Recoder
from nilearn.datasets.utils import _fetch_files, _get_dataset_dir
import pandas as pd
import requests
from sklearn.utils import Bunch
from abagen import io

WELL_KNOWN_IDS = Recoder(
    (('9861', 'H0351.2001', '178238387', '157722636'),
     ('10021', 'H0351.2002', '178238373', '157723301'),
     ('12876', 'H0351.1009', '178238359', '157722290'),
     ('15496', 'H0351.1015', '178238266', '162021642'),
     ('14380', 'H0351.1012', '178238316', '157721937'),
     ('15697', 'H0351.1016', '178236545', '157682966')),
    fields=('subj', 'uid', 'url', 't1w',)
)

VALID_DONORS = sorted(WELL_KNOWN_IDS.value_set('subj') |
                      WELL_KNOWN_IDS.value_set('uid'))


def fetch_microarray(data_dir=None, donors=['9861'], resume=True, verbose=1,
                     convert=True):
    """
    Downloads the Allen Human Brain Atlas microarray expression dataset

    Parameters
    ----------
    data_dir : str, optional
        Directory where data should be downloaded and unpacked. Default:
        current directory
    donors : list, optional
        List of donors to download; can be either donor number or UID. Can also
        specify 'all' to download all available donors. Default: 9861
    resume : bool, optional
        Whether to resume download of a partly-downloaded file. Default: True
    verbose : int, optional
        Verbosity level (0 means no message). Default: 1
    convert : bool, optional
        Whether to convert downloaded CSV files into parquet format for faster
        loading in the future; only available if ``fastparquet`` and ``python-
        snappy`` are installed. Default: True

    Returns
    -------
    data : :class:`sklearn.utils.Bunch`
        Dictionary-like object with keys ['microarray', 'ontology', 'pacall',
        'probes', 'annotation'], where corresponding values are lists of
        filepaths to downloaded CSV files.

    References
    ----------
    Hawrylycz, M. J., Lein, E. S., Guillozet-Bongaarts, A. L., Shen, E. H., Ng,
    L., Miller, J. A., ... & Abajian, C. (2012). An anatomically comprehensive
    atlas of the adult human brain transcriptome. Nature, 489(7416), 391.
    """

    url = "http://human.brain-map.org/api/v2/well_known_file_download/{}"

    dataset_name = 'allenbrain'
    if data_dir is None:
        data_dir = os.getcwd()
    data_dir = _get_dataset_dir(dataset_name, data_dir=data_dir,
                                verbose=verbose)

    sub_files = ('MicroarrayExpression.csv',
                 'Ontology.csv', 'PACall.csv',
                 'Probes.csv', 'SampleAnnot.csv')
    n_files = len(sub_files)

    if donors is not None and (isinstance(donors, (list, tuple))):
        for n, sub_id in enumerate(donors):
            if sub_id not in VALID_DONORS:
                raise ValueError('You provided invalid subject id {0} in a'
                                 'list. Subjects must be selected in {1}.'
                                 .format(sub_id, VALID_DONORS))
            donors[n] = WELL_KNOWN_IDS[sub_id]  # convert to ID system
    elif donors == 'all':
        donors = WELL_KNOWN_IDS.value_set('subj')
    else:
        donors = []
    donors = sorted(set(donors), key=lambda x: int(x))  # avoid duplicates

    files = [
         (os.path.join('normalized_microarray_donor{}'.format(sub), fname),
          url.format(WELL_KNOWN_IDS.url[sub]),
          dict(uncompress=True,
               move=os.path.join('normalized_microarray_donor{}'.format(sub),
                                 'donor{}.zip'.format(sub))))
         for sub in donors
         for fname in sub_files
    ]

    files = _fetch_files(data_dir, files, resume=resume, verbose=verbose)

    # if we want to convert files to parquet format it's good to do that now
    # this step is _already_ super long, so an extra 1-2 minutes is negligible
    if convert and io.use_parq:
        for fn in files[0::n_files] + files[2::n_files]:
            io._make_parquet(fn, convert_only=True)

    return Bunch(
        microarray=files[0::n_files],
        ontology=files[1::n_files],
        pacall=files[2::n_files],
        probes=files[3::n_files],
        annotation=files[4::n_files]
    )


def fetch_mri(data_dir=None, donors=['9861'], resume=True, verbose=1):
    """
    Downloads the Allen Human Brain Atlas T1w MRI images

    Parameters
    ----------
    data_dir : str, optional
        Directory where data should be downloaded and unpacked. Default:
        current directory
    donors : list, optional
        List of donors to download; can be either donor number or UID.
        Default: donor9861
    resume : bool, optional
        Whether to resume download of a partly-downloaded file. Default: True
    verbose : int, optional
        Verbosity level (0 means no message). Default: 1
    """

    raise NotImplementedError


def _fetch_alleninf_coords(*args, **kwargs):
    """
    Gets updated MNI coordinates for AHBA samples, as shipped with `alleninf`

    Returns
    -------
    coords : :class:`pandas.DataFrame`
        Updated MNI coordinates for all AHBA samples

    References
    ----------
    Updated MNI coordinates taken from https://github.com/chrisfilo/alleninf,
    which is licensed under the BSD-3 (reproduced here):

    Copyright (c) 2018, Krzysztof Gorgolewski
    All rights reserved.

    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.

    * Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.

    * Neither the name of the copyright holder nor the names of its
    contributors may be used to endorse or promote products derived from
    this software without specific prior written permission.

    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
    AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
    IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
    ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
    LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
    CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
    SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
    INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
    CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
    ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
    POSSIBILITY OF SUCH DAMAGE.
    """

    url = ("https://raw.githubusercontent.com/chrisfilo/alleninf/"
           "e48cd817849be4195b1569b7ac1eaf2c3e5a1085/alleninf/data/"
           "corrected_mni_coordinates.csv")

    with requests.get(url, stream=True) as r:
        coords = StringIO(r.content.decode('utf-8'))

    coords = pd.read_csv(coords).rename(dict(corrected_mni_x='mni_x',
                                             corrected_mni_y='mni_y',
                                             corrected_mni_z='mni_z'),
                                        axis=1)
    return coords.set_index('well_id')


def fetch_desikan_killiany(*args, **kwargs):
    """
    Fetches Desikan-Killiany atlas shipped with `abagen`

    Returns
    -------
    atlas : :class:`sklearn.utils.Bunch`
        Dictionary-like object with attributes ['image', 'info'] pointing to
        atlas image (.nii.gz) and information (.csv) files

    References
    ----------
    Desikan, R. S., Ségonne, F., Fischl, B., Quinn, B. T., Dickerson, B. C.,
    Blacker, D., ... & Albert, M. S. (2006). An automated labeling system
    for subdividing the human cerebral cortex on MRI scans into gyral based
    regions of interest. Neuroimage, 31(3), 968-980.
    """
    # grab resource filenames
    image = resource_filename('abagen', 'data/atlas-desikankilliany.nii.gz')
    info = resource_filename('abagen', 'data/atlas-desikankilliany.csv')

    return Bunch(image=image, info=info)



# -*- coding: utf-8 -*-
"""
Functions for cleaning and processing the AHBA microarray dataset
"""
import itertools
from nibabel.volumeutils import Recoder
import numpy as np
import pandas as pd
from abagen import io, utils
from abagen.datasets import _fetch_alleninf_coords

# AHBA structure IDs corresponding to different brain parts
ONTOLOGY = Recoder(
    (('4008', 'cerebral cortex', 'cortex'),
     ('4275', 'cerebral nuclei', 'subcortex'),
     ('4391', 'diencephalon', 'subcortex'),
     ('9001', 'mesencephalon', 'subcortex'),
     ('4696', 'cerebellum', 'cerebellum'),
     ('4833', 'metencephalon', 'cerebellum'),
     ('9512', 'myelencephalon', 'cerebellum')),
    fields=('id', 'name', 'structure')
)


def filter_probes(pacall, probes, threshold=0.5):
    """
    Performs intensity based filtering (IBF) of expression probes

    Uses binary indicator for expression levels in `pacall` to determine which
    probes have expression levels above background noise in `threshold` of
    samples across donors.

    Parameters
    ----------
    pacall : list
        List of PACall files from Allen Brain Institute. Optimally obtained by
        calling `abagen.fetch_microarray()` and accessing the `pacall`
        attribute on the resulting object
    probes : list
        List of probes files from Allen Brain Institute. Optimally obtained by
        calling `abagen.fetch_microarray()` and accessing the `probes`
        attribute on the resulting object
    threshold : (0, 1) float, optional
        Threshold for filtering probes. Specifies the proportion of samples for
        which a given probe must have expression levels above background noise.
        Default: 0.5

    Returns
    -------
    filtered : pandas.DataFrame
        Dataframe containing information on probes that should be retained
        according to intensity-based filtering
    """

    signal, samples = [], 0
    for fname in pacall:
        data = io.read_pacall(fname)
        samples += data.shape[-1]
        # sum binary expression indicator across samples for current subject
        signal.append(data.sum(axis=1).values)

    # calculate proportion of signal to noise for given probe across samples
    keep = (np.sum(signal, axis=0) / samples) > threshold

    # read in probe file (they're all the same) and drop "bad" probes
    filtered = io.read_probes(probes[0])[keep]

    return filtered


def get_stable_probes(microarray, annotation, probes):
    """
    Picks one probe to represent `microarray` data for each gene in `probes`

    If there are multiple probes with expression data for the same gene, this
    function will calculate the similarity of each probes' expression across
    donors and select the probe with the most consistent pattern of regional
    variation (i.e., "differential stability" or DS). Regions are
    operationalized by the "structure_id" column in `annotation`.

    Parameters
    ----------
    microarray : list of str
        List of microarray expression files from Allen Brain Institute.
        Optimally obtained by calling `abagen.fetch_microarray()` and accessing
        the `microarray` attribute on the resulting object.
    annotation : list of str
        List of annotation files from Allen Brain Institute. Optimally obtained
        by calling `abagen.fetch_microarray()` and accessing the `annotation`
        attribute on the resulting object.
    probes : pandas.DataFrame
        Dataframe containing information on probes that should be considered in
        representative analysis. Generally, intensity-based-filtering (i.e.,
        `probe_ibf()`) should have been used to reduce this list to only those
        probes with good expression signal

    Returns
    -------
    representative : pandas.DataFrame
        Dataframe containing information on probes that are most representative
        of their genes based on differential stability analysis

    References
    ----------
    Hawrylycz, M., Miller, J. A., Menon, V., Feng, D., Dolbeare, T., Guillozet-
    Bongaarts, A. L., ... & Lein, E. (2015). Canonical genetic signatures of
    the adult human brain. Nature Neuroscience, 18(12), 1832.
    """

    # read in microarray data for all subjects
    num_subj = len(microarray)

    # this is a relatively slow procedure (i.e., takes a couple of seconds)
    micro = [_reduce_micro(microarray[n], annotation[n], probes)
             for n in range(num_subj)]

    # get correlation of probe expression across samples for all donor pairs
    probe_corrs = np.zeros((len(probes), sum(range(num_subj))))
    for n, (s1, s2) in enumerate(itertools.combinations(range(num_subj), 2)):

        # find samples that current donor pair have in common
        samples = np.intersect1d(micro[s1].columns, micro[s2].columns)

        # the ranking process can take a few seconds on each loop
        # unfortunately, we have to do it each time because `samples` changes
        probe_corrs[:, n] = utils.efficient_corr(micro[s1][samples].T.rank(),
                                                 micro[s2][samples].T.rank())

    # group probes by gene and get probe corresponding to max correlation
    df = pd.DataFrame(dict(gene_symbol=probes.gene_symbol.values,
                           corrs=probe_corrs.mean(axis=1),
                           probe_id=probes.index))
    retained = (df.groupby('gene_symbol')
                  .apply(lambda x: x.loc[x.corrs.idxmax(), 'probe_id']))

    return probes[probes.index.isin(retained.values)]


def _reduce_micro(micro, annot, probes):
    """
    Collapse expression data in `micro` taken from same anatomical structure

    Parameters
    ----------
    micro : str
        Filepath to microarray file for single donor
    annot : str
        Filepath to annotation file for single donor; should be same donor as
        `micro`
    probes : pandas.DataFrame
        Probes that should be considered for analysis (i.e., those that made it
        through intensity-based filtering)

    Returns
    -------
    micro : (P x S) pd.core.frame.DataFrame
        Microarray expression data, where `P` is probes and `S` is anatomical
        structures
    """
    # read in microarray data and set columns as anatomical structure ID
    micro = io.read_microarray(micro)
    micro.columns = io.read_annotation(annot).structure_id

    # average across same structures
    micro = micro.groupby(micro.columns, axis=1).mean()

    # only keep good probes
    return micro[micro.index.isin(probes.index)]


def drop_mismatch_samples(annotation, ontology, corrected=True):
    """
    Parameters
    ----------
    annotation : str
        Annotation file from Allen Brain Institute. Optimally obtained by
        calling `abagen.fetch_microarray()` and accessing the `annotation`
        attribute on the resulting object
    ontology : str
        Ontology file from Allen Brain Institute. Optimally obtained by
        calling `abagen.fetch_microarray()` and accessing the `ontology`
        attribute on the resulting object
    corrected : bool, optional
        Whether to use the "corrected" MNI coordinates shipped with the
        `alleninf` package instead of the coordinates provided with the AHBA
        data. Default: True

    Returns
    -------
    keep : pandas.DataFrame
        Annotation data
    """

    # read in data files
    annot = io.read_annotation(annotation)
    ont = io.read_ontology(ontology)

    # add hemisphere + brain "structure" designation to annotation data
    hemi = dict(zip(ont.id, ont.hemisphere))
    stru = dict(zip(ont.id, ont.structure_id_path.apply(_get_path_structure)))
    annot = annot.assign(hemisphere=annot.structure_id.replace(hemi),
                         structure=annot.structure_id.replace(stru))

    # correct MNI coordinates, if requested
    if corrected:
        annot = _replace_mni_coords(annot)

    # only keep samples with consistent hemisphere + MNI coordinate designation
    keep = annot.query('(hemisphere == "L" & mni_x < 0) | '
                       '(hemisphere == "R" & mni_x > 0)')

    return keep


def _replace_mni_coords(annotation):
    """
    Replaces MNI coords in `annotation` with corrected coords from alleninf

    Parameters
    ----------
    annotation : str
        Annotation file from Allen Brain Institute. Optimally obtained by
        calling `abagen.fetch_microarray()` and accessing the `annotation`
        attribute on the resulting object

    Returns
    -------
    corrected : pandas.DataFrame
        Annotation data with corrected MNI coordinates
    """

    annotation = io.read_annotation(annotation)
    mni_coords = _fetch_alleninf_coords().loc[annotation.well_id]
    annotation[['mni_x', 'mni_y', 'mni_z']] = mni_coords.get_values()

    return annotation


def _get_path_structure(structure_path):
    """
    Gets overarching "structure" of region defined by `structure_path`

    Structure here is defined as one of ['cortex', 'subcortex', 'cerebellum']

    Parameters
    ----------
    structure_path : str
        As obtained from an ontology file from Allen Brain Institute. Optimally
        obtained by calling `abagen.fetch_microarray()`, loading the
        `ontology` attribute on the resulting object, and querying the
        `structure_id_path` column from the resulting dataframe

    Returns
    -------
    structure : str
        One of ['cortex', 'subcortex', 'cerebellum'] or None
    """
    structure_path = set(structure_path.split('/'))
    ids = list(ONTOLOGY.value_set('id').intersection(structure_path))

    try:
        return ONTOLOGY.structure[ids[0]]
    except IndexError:
        return


def normalize_expression(expression):
    """
    Performs scaled robust sigmoid (SRS) normalization on `expression` data

    Parameters
    ----------
    expression : (S, G) pandas.DataFrame
        Microarray expression data, where `S` is samples (or regions) and `G`
        is genes

    Returns
    -------
    normalized : (S, G) pandas.DataFrame
        Data from `expression` normalized separately for each gene
    """

    # get non-NaN values
    data = expression.dropna(axis=0, how='all').get_values()

    # calculate sigmoid normalization
    norm = (data - np.median(data, axis=0)) / np.std(data, axis=0)
    srs = 1 / (1 + np.exp(-norm))

    # rescale normalized values to a unit interval
    srs_min = srs.min(axis=0, keepdims=True)
    srs_max = srs.max(axis=0, keepdims=True)
    scaled = (srs - srs_min) / (srs_max - srs_min)

    # recreate dataframe and fill non-NaN values
    normalized = pd.DataFrame(np.nan,
                              columns=expression.columns,
                              index=expression.index)
    inds = expression[expression.notna().all(axis=1)].index
    normalized.loc[inds] = scaled

    return normalized



import numpy as np
import pandas as pd
import pytest
from abagen import utils
from abagen.datasets import fetch_desikan_killiany

ATLAS = fetch_desikan_killiany()
EXAMPLE_COORDS = dict(
    affine=np.array([[1, 0, 0, -90],
                     [0, 1, 0, -150],
                     [0, 0, 1, -80],
                     [0, 0, 0, 1]]),
    coords=[
        dict(
            ijk=[0, 0, 0],
            xyz=np.array([[-90, -150, -80]])
        ),
        dict(
            ijk=[[10, 10, 10], [100, 50, 100]],
            xyz=np.array([[-80, -140, -70], [10, -100, 20]])
        ),
        dict(
            ijk=[[54, 32, 20], [82, 205, 38], [32, 51, 82]],
            xyz=np.array([[-36, -118, -60], [-8, 55, -42], [-58, -99, 2]])
        )
    ]
)
EXAMPLE_METRICS = [
    dict(
        metric='mean',
        check=np.array([[1, 2], [3, 4], [5, 6]]),
        confirm=np.array([1.5, 3.5, 5.5]),
        kwargs=dict(axis=1)
    ),
    dict(
        metric='median',
        check=np.array([[1, 2, 3], [4, 5, 6], [7, 8, 9]]),
        confirm=np.array([2, 5, 8]),
        kwargs=dict(axis=1)
    )
]


def test_coords_transform():
    aff = EXAMPLE_COORDS['affine']
    for coords in EXAMPLE_COORDS['coords']:
        ijk, xyz = coords['ijk'], coords['xyz']
        assert np.all(utils.xyz_to_ijk(xyz, aff) == ijk)
        assert np.all(utils.ijk_to_xyz(ijk, aff) == xyz)
    with pytest.raises(ValueError):
        utils.xyz_to_ijk([[10, 10], [20, 30]], aff)
    with pytest.raises(ValueError):
        utils.ijk_to_xyz([[10, 10], [20, 30]], aff)


def test_check_atlas_info():
    # check appropriate usage
    out = utils.check_atlas_info(ATLAS.image, ATLAS.info)
    assert isinstance(out, pd.DataFrame)
    atlas_df = pd.read_csv(ATLAS.info)
    out = utils.check_atlas_info(ATLAS.image, atlas_df)
    atlas_df = atlas_df.set_index('id')
    out = utils.check_atlas_info(ATLAS.image, atlas_df)

    # check bad usage
    with pytest.raises(ValueError):
        utils.check_atlas_info(ATLAS.image, [1, 2, 3])

    bad_df = pd.DataFrame(columns=['id', 'hemisphere', 'structure'])
    with pytest.raises(ValueError):
        utils.check_atlas_info(ATLAS.image, bad_df)


def test_check_metric():
    for config in EXAMPLE_METRICS:
        metric = utils.check_metric(config['metric'])
        check, kwargs = config.get('check'), config.get('kwargs', {})
        assert np.allclose(metric(check, **kwargs), config['confirm'])

    # not an appropriate str
    with pytest.raises(ValueError):
        utils.check_metric('blargh')

    # doesn't accept axis kwarg
    with pytest.raises(TypeError):
        utils.check_metric(lambda x: np.mean(x))



# -*- coding: utf-8 -*-

import itertools
from nilearn._utils import check_niimg_3d
import numpy as np
import pandas as pd
from scipy.ndimage.measurements import center_of_mass
from scipy.spatial.distance import cdist
from scipy.stats import zscore

AGG_FUNCS = dict(
    mean=np.mean,
    median=np.median
)


def check_atlas_info(atlas, atlas_info):
    """
    Checks whether provided `info` on `atlas` is sufficient for processing

    Parameters
    ----------
    atlas : niimg-like object
        Parcel image, where each parcel should be identified with a unique
        integer ID
    atlas_info : str or pandas.DataFrame, optional
        Filepath to or pre-loaded dataframe containing information about
        `atlas`. Must jhave _at least_ columns 'id', 'hemisphere', and
        'structure' containing information mapping atlas IDs to hemisphere and
        broad structural class (i.e., "cortex", "subcortex", "cerebellum").
        Default: None

    Returns
    -------
    atlas_info : pandas.DataFrame
        Loaded dataframe with information on atlas

    Raises
    ------
    ValueError
        If `atlas_info` does not have sufficient information
    """
    atlas = check_niimg_3d(atlas)

    # load info, if not already
    if isinstance(atlas_info, str):
        atlas_info = pd.read_csv(atlas_info)

    if not isinstance(atlas_info, pd.DataFrame):
        raise ValueError('Provided `atlas_info` of type {} is not a filepath '
                         'or DataFrame. Please confirm inputs and try again.'
                         .format(type(atlas_info)))

    if 'id' in atlas_info.columns:
        atlas_info = atlas_info.set_index('id')

    ids = get_unique_labels(atlas)
    cols = ['hemisphere', 'structure']
    try:
        assert all(c in atlas_info.columns for c in cols)
        assert ('id' == atlas_info.index.name)
        assert np.setdiff1d(ids, atlas_info.index.values).size == 0
    except AssertionError:
        raise ValueError('Provided `atlas_info` does not have adequate '
                         'information  on supplied `atlas`. Please confirm '
                         'that `atlas_info` has columns [\'id\', '
                         '\'hemisphere\', \'structure\'], and that the region '
                         'IDs listed in `atlas_info` account for all those '
                         'found in `atlas.')

    return atlas_info


def check_metric(metric):
    """
    Confirms `metric` is a valid aggregation metric

    Parameters
    ----------
    metric : str or func, optional
        Mechanism by which to reduce arrays. If str, should be in ['mean',
        'median']. If func, should be able to accept an `N`-dimensional input
        and the `axis` keyword argument and return an `N-1`-dimensional output.
        Default: 'mean'

    Returns
    -------
    metric : func
        Mechanism by which to reduce arrays
    """
    if isinstance(metric, str):
        try:
            metric = AGG_FUNCS[metric]
        except KeyError:
            raise ValueError('Provided metric {0} is not valid. If supplied'
                             'as string, metric must be in {1}.'
                             .format(metric, list(AGG_FUNCS.keys())))
    else:
        try:
            test = np.random.rand(10, 10, 10)
            assert metric(test, axis=0).shape == (10, 10)
            assert isinstance(metric(test), float)
        except (AssertionError, TypeError):
            raise TypeError('Provided metric {0} does not perform as '
                            'expected. Please ensure it accepts the `axis` '
                            'keyword argument and can reduce an `N`-'
                            'dimensional input to an `N-1`-dimensional output.'
                            .format(metric))

    return metric


def efficient_corr(x, y):
    """
    Computes correlation of matching columns in `x` and `y`

    Parameters
    ----------
    x, y : (N, M) array_like
        Input data arrays

    Returns
    -------
    corr : (M,) numpy.ndarray
        Correlations of columns in `x` and `y`
    """
    corr = np.sum(zscore(x, ddof=1) * zscore(y, ddof=1), axis=0) / (len(x) - 1)

    return corr


def get_unique_labels(label_image):
    """
    Returns all possible ROI labels from ``label_image``

    Parameters
    ----------
    label_image : niimg-like object
        ROI image, where each ROI is identified with a unique integer ID

    Returns
    -------
    labels : np.ndarray
        Integer labels of all ROIS found within ``label_image``
    """

    label_image = check_niimg_3d(label_image)
    return np.trim_zeros(np.unique(label_image.get_data())).astype(int)


def get_centroids(label_image, labels_of_interest=None, image_space=False):
    """
    Finds centroids of ``labels_of_interest`` in ``label_image``

    Parameters
    ----------
    label_image : niimg-like object
        3D image containing integer label at each point
    labels_of_interest : array_like, optional
        List of values containing labels of which to find centroids. Default:
        all possible labels
    image_space : bool, optional
        Whether to return xyz (image space) coordinates for centroids based
        on transformation in ``label_image.affine``. Default: False

    Returns
    -------
    centroids : (N, 3) np.ndarray
        Coordinates of centroids for ROIs in input data
    """

    label_image = check_niimg_3d(label_image)

    # if no labels of interest provided, get all possible labels
    if labels_of_interest is None:
        labels_of_interest = get_unique_labels(label_image)

    # get centroids for all possible labels
    image_data = label_image.get_data()
    centroids = np.row_stack([center_of_mass(image_data == label) for
                              label in labels_of_interest])

    # return xyz if desired; otherwise, ijk
    if image_space:
        return ijk_to_xyz(centroids, label_image.affine)

    return centroids


def closest_centroid(coord, centroids, return_dist=False):
    """
    Returns index of `centroids` closest to `coord`

    Computes Euclidean distances between `coord` and each of `centroids`,
    returning index of closest centroid

    Parameters
    ----------
    coord : (1, 3) array_like
        Coordinates of sample
    centroids : (N, 3) array_like
        Centroids of parcels (in same space as `coord`)
    return_dist : bool, optional
        Whether to also return distance of closest centroid

    Returns
    -------
    closest : int
        Index of closest centroid in `centroids` to `coord`
    distance : float
        Distance of closest centroid in `centroids` to `coord`. Only returned
        if `return_dist=True`
    """

    distances = np.squeeze(cdist(np.atleast_2d(coord), centroids))
    closest = distances.argmin(axis=0)

    if return_dist:
        return closest, distances[closest]

    return closest


def _check_coord_inputs(coords):
    """
    Confirms `coords` are appropriate shape for coordinate transform

    Parameters
    ----------
    coords : array-like

    Returns
    -------
    coords : (3, N) np.ndarray
    """
    coords = np.atleast_2d(coords).T
    if 3 not in coords.shape:
        raise ValueError('Input coordinates must be of shape (3 x N). '
                         'Provided coordinate shape: {}'.format(coords.shape))
    if coords.shape[0] != 3:
        coords = coords.T
    # add constant term to coords to make 4 x N
    coords = np.row_stack([coords, np.ones_like(coords[0])])
    return coords


def ijk_to_xyz(coords, affine):
    """
    Converts `coords` in cartesian space to `affine` space

    Parameters
    ----------
    coords : (N, 3) array_like
        Cartesian (ijk) coordinate values
    affine : (4, 4) array_like
        Affine matrix containing displacement + boundary

    Returns
    -------
    xyz : (N, 3) np.ndarray
        Provided ``coords`` in ``affine`` space
    """
    coords = _check_coord_inputs(coords)
    aff_coords = np.dot(affine, coords)[:3].T
    return aff_coords


def xyz_to_ijk(coords, affine):
    """
    Converts `coords` in `affine` space to cartesian space

    Parameters
    ----------
    coords : (N, 3) array_like
        Image coordinate (xyz) values
    affine : (4, 4) array_like
        Affine matrix containing displacement + boundary

    Returns
    ------
    ijk : (N, 3) numpy.ndarray
        Provided `coords` in cartesian space
    """
    coords = _check_coord_inputs(coords)
    ijk_coords = np.linalg.solve(affine, coords)[:3].T.astype(int)
    return ijk_coords


def expand_roi(coords, dilation=1, return_array=True):
    """
    Expands coordinates ``coords`` to include neighboring coordinates

    Computes all possible coordinates of distance ``dilation`` from ``coords``.
    Returns a generator (``itertools.product``) by default, but can return an
    array if desired (if ``return_array=True``).

    Parameters
    ----------
    coords : (1, 3) array_like
        List of ijk values for coordinate in 3D space
    dilation : int, optional
        How many neighboring voxels to expand around `coords`. Default: 1
    return_array : bool, optional
        Whether to return array instead of generator. Default: True

    Returns
    -------
    coords : (27, 3) generator
        Coordinates of expanded ROI
    """

    def expand(x, d=1):
        return np.arange((x - d), (x + d + 1), dtype=int)

    # return all combinations of coordinates
    coords = np.squeeze(coords)
    gen = itertools.product(*[expand(n, d=dilation) for n in coords])

    # coerce to array if desired
    if return_array:
        return np.asarray(list(gen))

    return gen



# -*- coding: utf-8 -*-
"""
Functions for mapping AHBA microarray dataset to atlases and and parcellations
in MNI space
"""
from functools import reduce
from nilearn._utils import check_niimg_3d
import numpy as np
import pandas as pd
from scipy.spatial.distance import cdist
from sklearn.utils import Bunch
from abagen import io, process, utils


def _assign_sample(sample, atlas, sample_info=None, atlas_info=None,
                   tolerance=3):
    """
    Determines what ROI `sample` belongs to in `atlas`

    Parameters
    ----------
    sample : (1, 3) array_like
        Coordinates (ijk) of microarray probe in `atlas` space
    atlas : niimg-like object
        ROI image, where each ROI should be identified with a unique
        integer ID
    tolerance : int, optional
        Distance that sample must be from ROI for it to be considered within
        that ROI. This is only used if the sample is not directly inside a ROI.
        Default: 3mm

    Returns
    -------
    label : int
        ROI label of `sample`
    """

    # pull relevant info from atlas
    label_data = check_niimg_3d(atlas).get_data()

    # expand provided coordinates to include those w/i `tolerance` of `coords`
    # set a hard euclidean distance limit to account for different voxel sizes
    coords = utils.expand_roi(sample, dilation=tolerance, return_array=True)
    coords = coords[cdist(sample, coords).squeeze() < tolerance]

    # grab non-zero labels for expanded coordinates
    possible_labels = label_data[coords[:, 0], coords[:, 1], coords[:, 2]]
    nz_labels = possible_labels[possible_labels.nonzero()]
    labels, counts = np.unique(nz_labels, return_counts=True)

    # if atlas_info and sample_info are provided, drop potential labels who
    # don't match hemisphere or structural class defined in `sample_info`
    if atlas_info is not None and sample_info is not None:
        for old_label in labels:
            new_label = _check_label(old_label, sample_info, atlas_info)
            nz_labels[nz_labels == old_label] = new_label
        labels, counts = np.unique(nz_labels, return_counts=True)

    # if there is still nothing in the vicinity, return 0
    if labels.size == 0:
        return 0
    # if there is only one ROI in the vicinity, use that
    elif labels.size == 1:
        return labels[0]

    # if more than one ROI in the vicinity, return the most frequent
    indmax, = np.where(counts == counts.max())
    if indmax.size == 1:
        return labels[indmax[0]]

    # if two or more parcels tied for neighboring frequency, use ROI
    # with closest centroid to `coords`
    centroids = utils.get_centroids(atlas, labels)
    return labels[utils.closest_centroid(sample, centroids)]


def _check_label(label, sample_info, atlas_info):
    """
    Checks that `label` defined by `sample_info` is coherent with `atlas_info`

    Parameters
    ----------
    label : int
        Tenative label for sample in `sample_info`
    sample_info : pandas.DataFrame
    atlas_info : pandas.DataFrame

    Returns
    -------
    label : int
        New tentative label
    """

    cols = ['hemisphere', 'structure']

    if label != 0:
        sample_info = sample_info[cols]
        atlas_info = atlas_info.loc[label][cols]
        if not np.all(sample_info.values == atlas_info.values):
            label = 0

    return label


def label_samples(annotation, atlas, atlas_info=None, tolerance=2):
    """
    Matches all samples in `annotation` to closest ROIs in `atlas_info`

    Attempts to place each sample provided in `annotation` into a labelled ROI
    in `atlas_info`, where the latter is a 3D niimg-like object that contains
    ROI data, each with a unique integer ID.

    The function tries to best match the microarray sample to the ROIs by:
    1. Determining if the sample falls directly within a ROI.
    2. Checking to see if there are nearby ROIs by slowly expanding radius
       around sample (up to radius = `tolerance`); if there are multiple nearby
       ROIs, determining which ROI is closer (i.e., by centroid of the ROI).
    3. If `user_centroids` is set, find the ROI with the closest centroid

    If there is still no ROI for a given sample, it is assigned as "NA" and
    should likely be ignored in further analyses.

    Parameters
    ----------
    annotation : (S, 13) pandas.DataFrame
        Annotation information, where `S` is samples
    atlas : niimg-like object
        ROI image, where each ROI should be identified with a unique integer ID
    atlas_info : pandas.DataFrame, optional
        Filepath to or pre-loaded dataframe containing information about
        `atlas`. Must have _at least_ columns 'id', 'hemisphere', and
        'structure' containing information mapping atlas IDs to hemisphere and
        broad structural class (i.e., "cortex", "subcortex", "cerebellum").
        Default: None
    tolerance : int, optional
        Distance (in mm) that a sample must be from an ROI boundary for it to
        be considered within that ROI. This is only used if the sample is not
        directly within a ROI. Default: 2

    Returns
    -------
    labels : (S, 1) pandas.DataFrame
        Dataframe with ROI labels
    """

    # get annotation and atlas data
    annotation = io.read_annotation(annotation)
    atlas = check_niimg_3d(atlas)
    label_data, affine = atlas.get_data(), atlas.affine

    # load atlas_info, if provided
    if atlas_info is not None:
        atlas_info = utils.check_atlas_info(atlas, atlas_info)

    # get ijk coordinates for microarray samples and find labels
    g_ijk = utils.xyz_to_ijk(annotation[['mni_x', 'mni_y', 'mni_z']], affine)
    labelled_samples = label_data[g_ijk[:, 0], g_ijk[:, 1], g_ijk[:, 2]]

    # if sample coordinates aren't directly inside a parcel, increment radius
    # around sample up to `tolerance` to try and find nearby parcels.
    # if still no parcel, then ignore this sample
    for idx in np.where(labelled_samples == 0)[0]:
        label, tol = labelled_samples[idx], 1
        while label == 0 and tol <= tolerance:
            label = _assign_sample(g_ijk[[idx]], atlas,
                                   sample_info=annotation.iloc[idx],
                                   atlas_info=atlas_info,
                                   tolerance=tol)
            tol += 1
        labelled_samples[idx] = label

    return pd.DataFrame(labelled_samples, dtype=int,
                        columns=['label'], index=annotation.index)


def group_by_label(microarray, sample_labels, labels=None, metric='mean'):
    """
    Averages expression data in `microarray` over samples with same label

    Parameters
    ----------
    microarray : (S, G) pandas.DataFrame
        Microarray expression data, where `S` is samples and `G` is genes
    sample_labels : (S, 1) pandas.DataFrame
        ROI labels for `S` samples, as returned by e.g., `label_samples()`
    labels : (L,) array_like, optional
        All possible labels for parcellation scheme (to account for possibility
        that some parcels have NO expression data). Default: None
    metric : str or func
        Mechanism by which to collapse across samples within an ROI. If str,
        should be in ['mean', 'median'].

    Returns
    -------
    gene_by_label : (L, G) pandas.DataFrame
        Microarray expression data
    """

    # get combination function
    metric = utils.check_metric(metric)

    # get missing labels
    if labels is not None:
        missing = np.setdiff1d(labels, sample_labels)
        labels = pd.DataFrame(columns=microarray.columns,
                              index=pd.Series(missing, name='label'))

    gene_by_label = (microarray.merge(sample_labels,
                                      left_index=True, right_index=True)
                               .groupby('label')
                               .aggregate(metric)
                               .append(labels)
                               .drop([0])
                               .sort_index()
                               .rename_axis('label'))

    return gene_by_label


def get_expression_data(files, atlas, atlas_info=None, *,
                        exact=True, tolerance=2, metric='mean',
                        ibf_threshold=0.5, corrected_mni=True,
                        return_counts=False):
    """
    Assigns microarray expression data in `files` to ROIs defined in `atlas`

    Parameters
    ----------
    files : dict
        Optimally obtained by calling `abagen.fetch_microarray()`, this should
        be a dict with keys ['annotation', 'microarray', 'ontology', 'pacall',
        'probes'], where corresponding values are lists of filepaths to
        downloaded AHBA data
    atlas : niimg-like object
        A parcellation image in MNI space, where each parcel is identified by a
        unique integer ID
    atlas_info : str or :class:`pandas.DataFrame`, optional
        Filepath to or pre-loaded dataframe containing information about
        `atlas`. Must have at least columns 'id', 'hemisphere', and 'structure'
        containing information mapping atlas IDs to hemisphere (i.e, "L", "R")
        and broad structural class (i.e., "cortex", "subcortex", "cerebellum").
        Default: None
    exact : bool, optional
        Whether to use exact matching of donor tissue samples to parcels in
        `atlas`. If True, this function will match tissue samples to parcels
        within `threshold` mm of the sample; any samples that are beyond
        `threshold` mm of a parcel will be discarded. This may result in some
        parcels having no assigned sample / expression data. If False, the
        default matching procedure will be performed and followed by a check
        for parcels with no assigned samples; any such parcels will be matched
        to the nearest sample (nearest defined as the sample with the closest
        Euclidean distance to the parcel centroid). Default: True
    tolerance : int, optional
        Distance (in mm) that a sample must be from a parcel for it to be
        matched to that parcel. Default: 3
    metric : str or func, optional
        Mechanism by which to collapse across donors, if input `files` provides
        multiple donor datasets. If a str, should be in ['mean', 'median']; if
        a function, should be able to accept an `N`-dimensional input and the
        `axis` keyword argument and return an `N-1`-dimensional output.
        Default: 'mean'
    ibf_threshold : [0, 1] float, optional
        Threshold for intensity-based filtering specifying. This number should
        specify the ratio of samples, across all supplied donors, for which a
        probe must have signal above background noise in order to be retained.
        Default: 0.5
    corrected_mni : bool, optional
        Whether to use the "corrected" MNI coordinates shipped with the
        `alleninf` package instead of the coordinates provided with the AHBA
        data when matching tissue samples to anatomical regions. Default: True
    return_counts : bool, optional
        Whether to return how many samples were assigned to each parcel in
        `atlas` for each donor. Default: False

    Returns
    -------
    expression : (R x G) :class:`pandas.DataFrame`
        Microarray expression for `R` regions in `atlas` across `G` genes,
        averaged across donors.
    counts : :class:`pandas.DataFrame`
        Number of samples assigned to each parcel in `atlas` (by donor, if
        multiple donors provided); only provided if `return_counts=True`
    """

    # coerce to Bunch in case a simple dictionary was provided
    files = Bunch(**files)
    for key in ['microarray', 'probes', 'annotation', 'pacall', 'ontology']:
        if key not in files:
            raise KeyError('Provided `files` dictionary is missing {}. '
                           'Please check inputs.'.format(key))

    # load atlas_info, if provided
    atlas = check_niimg_3d(atlas)
    if atlas_info is not None:
        atlas_info = utils.check_atlas_info(atlas, atlas_info)

    # get combination functions
    metric = utils.check_metric(metric)

    # get some info on the number of subjects, labels in `atlas_img`
    num_subj = len(files.microarray)
    all_labels = utils.get_unique_labels(atlas)
    if not exact:
        centroids = utils.get_centroids(atlas, labels_of_interest=all_labels)

    probes = process.filter_probes(files.pacall, files.probes,
                                   threshold=ibf_threshold)
    probes = process.get_stable_probes(files.microarray, files.annotation,
                                       probes)

    expression, counts = [], np.zeros((len(all_labels) + 1, num_subj))
    missing = []
    for subj in range(num_subj):
        # get rid of samples whose coordinates don't match ontological profile
        annotation = process.drop_mismatch_samples(files.annotation[subj],
                                                   files.ontology[subj],
                                                   corrected=corrected_mni)

        # get representative probes + samples from microarray data
        microarray = io.read_microarray(files.microarray[subj])
        samples = microarray.loc[probes.index, annotation.index].T
        samples.columns = probes.gene_symbol

        # assign samples to regions and aggregate samples w/i the same region
        sample_labels = label_samples(annotation, atlas,
                                      atlas_info=atlas_info,
                                      tolerance=tolerance)
        expression += [group_by_label(samples, sample_labels,
                                      all_labels, metric=metric)]

        # get counts of samples collapsed into each ROI
        labs, num = np.unique(sample_labels, return_counts=True)
        counts[labs, subj] = num

        if not exact:
            coords = utils.xyz_to_ijk(annotation[['mni_x', 'mni_y', 'mni_z']],
                                      atlas.affine)
            empty = np.setdiff1d(all_labels, labs)
            closest, dist = utils.closest_centroid(coords, centroids[empty],
                                                   return_dist=True)
            closest = samples.loc[annotation.iloc[closest].index]
            closest.index = pd.Series(empty, name='label')
            missing += [(closest, dict(zip(empty, np.diag(dist))))]

    # check for missing ROIs and fill in, as needed
    if not exact:
        empty = reduce(set.intersection, [set(f.index) for f, d in missing])
        for roi in empty:
            ind = np.argmin([d.get(roi) for f, d in missing])
            expression[ind].loc[roi] = missing[ind][0].loc[roi]
            counts[roi, ind] += 1

    # normalize data and aggregate across donors
    expression = [process.normalize_expression(e) for e in expression]
    expression = pd.concat(expression).groupby('label').aggregate(metric)

    if return_counts:
        return expression, counts[1:]

    return expression



__all__ = ['__version__', 'get_expression_data',
           'fetch_microarray', 'fetch_desikan_killiany', 'io']

from .info import __version__

from abagen.allen import get_expression_data
from abagen.datasets import fetch_microarray, fetch_desikan_killiany
from abagen import io



import pytest
from abagen.datasets import fetch_microarray


@pytest.fixture(scope='session')
def testdir(tmpdir_factory):
    data_dir = tmpdir_factory.mktemp('data')
    return data_dir


@pytest.fixture(scope='session')
def testfiles(testdir):
    files = fetch_microarray(data_dir=str(testdir),
                             donors=['12876', '15496'],
                             convert=True)
    return files



# -*- coding: utf-8 -*-

__version__ = '0.0.1'

NAME = 'abagen'
MAINTAINER = 'Ross Markello'
EMAIL = 'rossmarkello@gmail.com'
VERSION = __version__
LICENSE = 'BSD-3'
DESCRIPTION = """\
A toolbox for working with the Allen Brain Atlas human genetic data\
"""
LONG_DESCRIPTION = 'README.rst'
LONG_DESCRIPTION_CONTENT_TYPE = 'text/x-rst'
URL = 'https://github.com/rmarkello/{name}'.format(name=NAME)
DOWNLOAD_URL = ('http://github.com/rmarkello/{name}/archive/{ver}.tar.gz'
                .format(name=NAME, ver=__version__))

INSTALL_REQUIRES = [
    'nibabel',
    'nilearn',
    'numpy',
    'pandas',
    'requests',
    'scikit-learn',
    'scipy',
]

TESTS_REQUIRES = [
    'codecov',
    'pytest',
    'pytest-cov'
]

EXTRAS_REQUIRES = {
    'doc': [
        'sphinx>=1.2',
        'sphinx_rtd_theme'
    ],
    'io': [
        'fastparquet',
        'python-snappy'
    ],
    'tests': TESTS_REQUIRES
}

EXTRAS_REQUIRES['all'] = list(set([
    v for deps in EXTRAS_REQUIRES.values() for v in deps
]))


PACKAGE_DATA = {
    'abagen': [
        'data/*'
        ]
}

CLASSIFIERS = [
    'Development Status :: 3 - Alpha',
    'Intended Audience :: Science/Research',
    'License :: OSI Approved :: BSD License',
    'Programming Language :: Python :: 3.6',
]



import pandas as pd
import pytest
from abagen import allen
from abagen.datasets import fetch_desikan_killiany

ATLAS = fetch_desikan_killiany()


def test_vanilla_get_expression_data(testfiles):
    out = allen.get_expression_data(testfiles, ATLAS.image)
    assert isinstance(out, pd.DataFrame)
    assert out.index.name == 'label'
    assert out.columns.name == 'gene_symbol'

    with pytest.raises(KeyError):
        allen.get_expression_data({'microarray': [1, 2, 3]}, ATLAS.image)


def test_extra_get_expression_data(testfiles):
    for opts in [{'atlas_info': ATLAS.info},
                 {'exact': False},
                 {'atlas_info': ATLAS.info, 'exact': False}]:
        out = allen.get_expression_data(testfiles, ATLAS.image, **opts)
        assert isinstance(out, pd.DataFrame)
        assert out.index.name == 'label'
        assert out.columns.name == 'gene_symbol'



#!/usr/bin/env python

import os
import sys


def main():
    from setuptools import setup, find_packages

    if sys.version_info < (3, 6):
        raise SystemError('You need Python >=3.6 or above to use abagen.')

    # get package information
    ldict = locals()
    curr_path = os.path.dirname(__file__)
    with open(os.path.join(curr_path, 'abagen', 'info.py')) as infofile:
        exec(infofile.read(), globals(), ldict)

    # get long description from README
    with open(os.path.join(curr_path, ldict['LONG_DESCRIPTION'])) as src:
        ldict['LONG_DESCRIPTION'] = src.read()

    setup(
        classifiers=ldict['CLASSIFIERS'],
        description=ldict['DESCRIPTION'],
        download_url=ldict['DOWNLOAD_URL'],
        extras_require=ldict['EXTRAS_REQUIRES'],
        install_requires=ldict['INSTALL_REQUIRES'],
        license=ldict['LICENSE'],
        long_description=ldict['LONG_DESCRIPTION'],
        long_description_content_type=ldict['LONG_DESCRIPTION_CONTENT_TYPE'],
        maintainer=ldict['MAINTAINER'],
        maintainer_email=ldict['EMAIL'],
        name=ldict['NAME'],
        packages=find_packages(exclude=['abagen/tests']),
        package_data=ldict['PACKAGE_DATA'],
        tests_require=ldict['TESTS_REQUIRES'],
        url=ldict['URL'],
        version=ldict['VERSION'],
    )


if __name__ == '__main__':
    main()
